# _*_ coding:utf-8 _*_

"""
    App loads the authentication record of the project
    The mapping of the main path of App
"""
from Xeu.core.route.RouterGroup import RouterGroup

""" App Register """

INSTALL_APP = [
    'AdminSystemIndexHtml',
    'BaseDemo',
    'AdminApi'
]

"""
    Prefix routing for App main routes、
"""

admin_route_group = RouterGroup(
    group='admin',
    route_map=dict(
        AdminSystemIndexHtml='index',
        AdminApi='api'
    )
)
example_route_group = RouterGroup(
    group='example',
    route_map=dict(
        BaseDemo='base',
    )
)

SETUP_MAIN_ROUTERS = (admin_route_group + example_route_group).to_dict()

"""
    APP ROOT DIR
"""

APP_ROOT = 'app'




# _*_ coding:utf-8 _*_
from abc import abstractmethod, ABCMeta


class TaskClient(metaclass=ABCMeta):

    @abstractmethod
    def create_task(self, *args, **kwargs):
        pass

    @abstractmethod
    def stop_task(self, *args, **kwargs):
        pass

    @abstractmethod
    def query_task(self, *args, **kwargs):
        pass

    @abstractmethod
    def list_tasks(self, *args, **kwargs):
        pass


class TaskServer(metaclass=ABCMeta):

    @abstractmethod
    def init_task_server(self, *args, **kwargs):
        """

        :param args:
        :param kwargs:
        :return:
        """
        #
        pass

    @abstractmethod
    def stop_task_server(self, *args, **kwargs):
        pass

    @abstractmethod
    def start_task_server(self, *args, **kwargs):
        pass



# -*- coding: utf-8 -*-
"""
    常用异常字段
"""
from tornado.web import MissingArgumentError
from pymysql.err import DataError # as pymysql
__all__ = ['TokenRefreshError', 'ParameterError', 'DataBaseOperationalError', 'ModuleAuthError', 'MissValueError',
           'SettlementError', 'ConstError', 'ConstCaseError', 'MissingArgumentError', 'DataError']


class TokenRefreshError(Exception):
    """
    token 刷新异常
    """
    def __init__(self, errmsg=None, errcode=None):
        Exception.__init__(self)
        self.message = errmsg
        self.code = errcode

    def __str__(self):
        return self.message, self.code


class FormError(Exception):
    """
    参数错误
    """
    def __init__(self, errmsg=None, errcode=None):
        Exception.__init__(self)
        self.message = errmsg
        self.code = errcode

    def __str__(self):
        return self.message, self.code


class ParameterError(Exception):
    """
    参数错误
    """
    def __init__(self, errmsg=None, errcode=None):
        Exception.__init__(self)
        self.message = errmsg
        self.code = errcode

    def __str__(self):
        return self.message, self.code


class DataBaseOperationalError(Exception):
    """
    数据库操作错误
    """
    def __init__(self, errmsg=None, errcode=None):
        Exception.__init__(self)
        self.message = errmsg
        self.code = errcode

    def __str__(self):
        return self.message, self.code


class ModuleAuthError(Exception):
    """
    模块访问权限
    """
    def __init__(self, error_info):
        Exception.__init__(self)
        self.message = error_info

    def __str__(self):
        return self.message


class MissValueError(Exception):
    """
    请求参数丢失
    """
    def __init__(self, error_info):
        Exception.__init__(self)
        self.message = error_info

    def __str__(self):
        return self.message


class SettlementError(Exception):
    """
    请求参数丢失
    """
    def __init__(self, error_info):
        Exception.__init__(self)
        self.message = error_info

    def __str__(self):
        return self.message


class ConstError(TypeError):
    """
    请求参数丢失
    """
    def __init__(self, error_info):
        Exception.__init__(self)
        self.message = error_info

    def __str__(self):
        return self.message


class ConstCaseError(ConstError):
    """
    请求参数丢失
    """
    def __init__(self, error_info):
        Exception.__init__(self)
        self.message = error_info

    def __str__(self):
        return self.message


# _*_ coding:utf-8 _*_
from abc import abstractmethod, ABCMeta
from Xeu.configure.base import ConfigBase


class ConfigFileOperation(metaclass=ABCMeta):

    def __init__(self, *args, **kwargs):
        pass

    @abstractmethod
    def write(self, *arg, **kwargs):
        raise NotImplementedError

    @abstractmethod
    def read(self, *arg, **kwargs) -> ConfigBase:
        raise NotImplementedError



# -*- coding: utf-8 -*-
from functools import wraps


def middleware(*middleware_args, **middleware_kwargs):
    # print(_func)
    # print(args)
    # print(middleware_kwargs)
    def _set_middleware(func):
        """
        中间件模块
        :param func:
        :return:
        """
        @wraps(func)
        def wrap(self, *args, **kwargs):
            """
            此处必须要有 *args, **kwargs 不然具名路由或者参数路由会出现问题
            :param self:
            :param args:
            :param kwargs:
            :return:
            """
            # 跨站域白名单

            import importlib
            # from Config.LoggingConf import Logger
            for middleware in middleware_args:
                _MiddlewaresSetup = importlib.import_module('Middlewares.'+middleware)
                _Middlewares = _MiddlewaresSetup.Core(_self=self, kwargs=kwargs)
                mid_res = _Middlewares.handle()
                if mid_res[0] is False:
                    self.write(mid_res[1])
                    self.finish()
                    return

            # _setting_headers = middleware_kwargs.get('set_header', SET_DEFAULT_HEADER)
            # if _setting_headers:
            #     for _setting, value in _setting_headers.items():
            #         self.set_header(_setting, value)
            try:
                # 管理端登录
                # Logger.info('method doc is %s' % func.__doc__)
                return func(self, *args, **kwargs)

            except Exception as e:
                # Logger.error('Service 执行 [状态] 失败 [详情] %s' % str(e))
                self.write('fail')
                self.finish()

        return wrap
    return _set_middleware



# _*_ coding:utf-8 _*_
import os
from Xeu.core.route.RouterCore import router_map_pool, api_doc
from Xeu.configure.init import Setup, BASE_DIR, LISTEN, APP_ROOT, INSTALL_APP, SETUP_MAIN_ROUTERS
from Xeu.logger.conf import Logger


"""
    The load file for the App
"""
app = None

apps_root = os.path.join(BASE_DIR, APP_ROOT)

_compile_lis = []


# todo INstall 与 路由映射的检查一致性
def is_app_consistence():
    app_consis = set(SETUP_MAIN_ROUTERS.keys()) ^ set(INSTALL_APP)
    if app_consis:
        return False, app_consis
    return True, None


def loading(root, app_root):
    """ 遍历目录下所有要导入的应用文件 """
    for app_file in os.listdir(root):
        app_file_path = os.path.join(root, app_file)
        if os.path.isdir(app_file_path) and not app_file.startswith('_'):
            """ dir """
            loading(app_file_path, app_root+'.'+app_file)
        elif not os.path.isdir(app_file_path):
            """ file """
            app_name = app_file.split('.')[0]
            if app_name in INSTALL_APP:

                # 一致性判断
                state, arg = is_app_consistence()
                if not state:
                    raise ValueError('Inconsistency between RouterMap and INSTALL_APP %s ' % str(arg))

                # todo 预加载 应用目录下的模块 此处 后面可以进行接口模块校验
                _compile_lis.append("from "+app_root + "."+str(app_name)+" import *")
            else:
                pass


"""
    Load the application in the App directory and load the routing map
"""
try:
    Logger.info('[system] loading component class of handler ......')
    loading(apps_root, app_root=APP_ROOT)
    for source in _compile_lis:
        try:
            eval(compile(source, '', 'exec'))
        except Exception as e:
            Logger.error(f'[system] loading component class of handler <{source}> [Fail] when {e} occurs as {source}')

    Logger.info('[system] loading component class of handler [OK]')
except ModuleNotFoundError as e:
    Logger.error(f'[system] loading component class of handler [Fail] {e}')
except ValueError as e:
    Logger.error(f'[system] loading component class of handler [Fail] {e}')
else:
    # sort router_map_pool
    router_map_pool.sort(key=lambda tup: tup[0], reverse=True)
    for router_map in router_map_pool:
        app.add_handlers(".*$", [router_map])
        Logger.info(f'[system] routing map loading <{router_map}> [OK]')

# def gen_pia_doc(api_doc_dic):
#     # print(api_doc_dic)
#     for full_class, api_doc in api_doc_dic.items():
#         # print(full_class, api_doc)
#         for method, method_doc in api_doc.items():
#             if method_doc is not None:
#
#                 print(method, method_doc.strip())
#
#
# gen_pia_doc(api_doc)
#
#
# if __name__ == '__main__':
#     print(is_app_consistence())
Setup(
    base_config=('base.toml', )
)



# _*_ coding:utf-8 _*_
import copy

__all__ = ['RouterGroup']


class RouterGroup(object):

    def __init__(self, group=None, route_map=None):

        self.route_map = route_map is not None and route_map or {}

        if isinstance(route_map, RouterGroup):
            if route_map.url_map:
                self.route_map = copy.deepcopy(route_map.url_map)
        elif isinstance(route_map, dict):
            if route_map == {}:
                raise ValueError('route_map must not be empty')
        else:
            raise ValueError('route_map must be dict')

        if self.route_map:
            for app, url in self.route_map.items():
                if url != '' and group not in [None, '']:
                    if url.startswith('/'):
                        self.__group = '/' + group
                    else:
                        self.__group = '/' + group + '/'
                    self.route_map[app] = self.__group + url
                else:
                    if url.startswith('/'):
                        self.route_map[app] = url
                    else:
                        self.route_map[app] = '/' + url

    def __add__(self, other):
        if not isinstance(other, RouterGroup) and not other.url_map:
            raise ValueError('type r-reference must be RouterGroup')
        mix = self.route_map.keys() & other.url_map.keys()
        if mix:
            raise ValueError('must not exist mix-key like %s' % str(mix))

        __add_dic = dict()
        __add_dic.update(other.url_map)
        __add_dic.update(self.route_map)
        return RouterGroup(route_map=__add_dic)

    @property
    def url_map(self):
        return self.route_map

    def __repr__(self):
        return str(self.route_map)

    def to_dict(self):
        return self.route_map




# coding=utf-8

from setuptools import setup, find_packages


def readme():
    with open('./README', encoding="utf-8") as f:
        _long_description = f.read()
        return _long_description

packages=find_packages()

print('packages:', packages)

setup(
    name='Xeu',
    version="0.0.1.17",
    description=(
        """
            Xeu
        """
    ),
    long_description=readme(),
    long_description_content_type="text/markdown",
    keywords=[],
    author='Jayson Leo',
    author_email='2835347017@qq.com',
    maintainer='Jayson Leo',
    maintainer_email='2835347017@qq.com',
    license='MIT License',

    platforms=["linux", 'windows'],
    classifiers=[
        'Development Status :: 4 - Beta',
        'Operating System :: OS Independent',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python',
        'Programming Language :: Python :: Implementation',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Topic :: Software Development :: Libraries'
    ],
    install_requires=[
        'toml',
        'Tornado',
    ],
    packages=packages,
    entry_points={
        'console_scripts': [
            'xeu=Xeu.setup.script.manage:run'
        ]
    }
)


import collections
from typing import Dict

import toml

from Xeu.configure.base import Config, ConfigBase
from Xeu.configure.collection import Collection
from Xeu.utils.config.base import ConfigFileOperation


class TomlConfigFileOperation(ConfigFileOperation):

    def __init__(self, config_path: str, collection: Collection, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.dic = dict()
        self.toml_file_path = config_path
        self.__collection = collection

    # def __add__(self, other):
    #     self.dic.update(self.other)
    #     return self.dic

    def write(self):
        mysql_dic = {"user": "root", "password": "Aa1234"}
        mysql2_dic = {"user1": "root", "password2": "Aa1234"}
        mysql_dic.update(mysql2_dic)
        with open(self.toml_file_path, "w", encoding="utf-8") as fs:
            toml.dump(mysql_dic, fs)

    def read(self) -> ConfigBase:
        with open(self.toml_file_path, "r", encoding="utf-8") as fs:
            t_data = toml.load(fs)
        return Config(t_data, self.__collection)






# _*_ coding:utf-8 _*_
"""
    该模块用于加载根配置文件
"""
import os
import collections
from copy import copy
from typing import Dict, Tuple, Union

from Xeu.configure.base import Config
from Xeu.configure.collection import Collector
from Xeu.utils.config.TomlTool import TomlConfigFileOperation
from Xeu.utils.config.base import ConfigFileOperation

BASE_CONFIG_FILE = ''
BASE_CONFIG_DIR = 'configure'
BASE_CONFIG_RAW_DATA = None
PORT = 8008
IPV4 = '0.0.0.0'

BASE_CONFIG_DATA: Dict

Collector.set('LISTEN', 'ipv4', IPV4)
Collector.set('LISTEN', 'port', PORT)

workspace_config = __import__('configure.AppConfig')
APP_ROOT = workspace_config.AppConfig.APP_ROOT
INSTALL_APP = workspace_config.AppConfig.INSTALL_APP
SETUP_MAIN_ROUTERS = workspace_config.AppConfig.SETUP_MAIN_ROUTERS

system_config = __import__('configure.SystemConfig')
COOKIE_SECRET = workspace_config.SystemConfig.COOKIE_SECRET
TEMPLATES_URL = workspace_config.SystemConfig.TEMPLATES_URL
STATIC_URL = workspace_config.SystemConfig.STATIC_URL
INDEX_FILE = workspace_config.SystemConfig.INDEX_FILE
BASE_DIR = workspace_config.SystemConfig.BASE_DIR


class _Setup(object):
    """
        配置框架所需的 端口，IP，日志等

    """

    def __init__(self):
        """

        :param base_config:
        """
        global BASE_DIR
        global LISTEN
        global BASE_CONFIG_RAW_DATA
        global BASE_CONFIG_FILE
        global BASE_CONFIG_DIR
        self.LISTEN = copy(LISTEN)
        self.BASE_DIR = copy(BASE_DIR)
        self.BASE_CONFIG_RAW_DATA = copy(BASE_CONFIG_RAW_DATA)
        self.BASE_CONFIG_FILE = copy(BASE_CONFIG_FILE)
        self.BASE_CONFIG_DIR = copy(BASE_CONFIG_DIR)

    def __resolve_base_config(self):

        self.BASE_CONFIG_FILE = self.base_config[0]
        if self.base_config.__len__() > 1:
            if issubclass(self.base_config[1], ConfigFileOperation):
                self.__base_config_handler = self.base_config[1]
            else:
                return ValueError("配置文件解析必须实现接口 ConfigFileOperation")

    def __load_base_data_config(self):
        self.BASE_CONFIG_RAW_DATA = self.__base_config_handler(
            os.path.join(self.BASE_DIR, self.BASE_CONFIG_DIR, self.BASE_CONFIG_FILE),
            Collector
        ).read()

    def __call__(self, base_config: Tuple[Union[str, ConfigFileOperation]]):
        self.base_config = base_config
        self.__base_config_handler = TomlConfigFileOperation
        self.__resolve_base_config()
        self.__load_base_data_config()


Setup = _Setup()
LISTEN = Collector.LISTEN



# _*_ coding:utf-8 _*_
import json
import tornado.web
import tornado.gen
from Xeu.core.route.RouterCore import app
from Xeu.utils.ResponseCodeManage import ResponseCode
from Xeu.utils.MiddlewareSetup import middleware
from Xeu.utils.CommonException import *
from Xeu.core.loader.__RootApplication import BaseHandlerCore
from Xeu.logger.conf import Logger
from concurrent.futures import ThreadPoolExecutor


